#include "StdAfx.h"
#include "HTTPClientCURL.h"

CMyHTTPClientCURL::CMyHTTPClientCURL(void)
	: m_pCURL(NULL)
	, m_dwLastHttpError(0)
	, m_pReadPosition(NULL)
{
	m_stChunk.memory = NULL; /* we expect realloc(NULL, size) to work */
	m_stChunk.size = 0;    /* no data at this point */

	m_pCURL = curl_easy_init();
}

CMyHTTPClientCURL::~CMyHTTPClientCURL(void)
{
	if (m_pCURL != NULL)
	{
		/* always cleanup */
		curl_easy_cleanup(m_pCURL);
	}

	ResetResponse();
}

int CMyHTTPClientCURL::OpenSession(LPCTSTR szAddress, USHORT uPort, LPCTSTR szUsername, LPCTSTR szPassword, UINT uiMsTimeout/*=DEFAULT_HTTPCONNECT_TIMEOUT*/)
{
	CString strUsrPwd;

	if (m_pCURL != NULL)
	{
		strUsrPwd.Format("%s:%s", szUsername, szPassword);

		curl_easy_setopt(m_pCURL, CURLOPT_USERPWD, strUsrPwd);

		curl_easy_setopt(m_pCURL, CURLOPT_TIMEOUT_MS, uiMsTimeout);
		curl_easy_setopt(m_pCURL, CURLOPT_CONNECTTIMEOUT_MS, uiMsTimeout);

		/* send all data to this function  */
		curl_easy_setopt(m_pCURL, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
		/* we pass our 'chunk' struct to the callback function */
		curl_easy_setopt(m_pCURL, CURLOPT_WRITEDATA, (void *)&m_stChunk);
	}

	m_sAddress = szAddress;
	m_iPort = uPort;
	m_strUsername= szUsername;
	m_strPassword= szPassword;

	return 0;
}

// Pide una conexion HTTP
int CMyHTTPClientCURL::Request(LPCTSTR szPath, UINT uiMsTimeout/*=DEFAULT_HTTPCURL_CONNECT_TIMEOUT*/)
{
	CURLcode curl_res;
	int iResult = 0;
	CString sURL;

	sURL.Format(_T("http://%s:%d"), m_sAddress, m_iPort);
	sURL.AppendFormat(szPath[0] == _T('/') ? _T("%s") : _T("/%s"), szPath);

	// Init variables for the new request
	curl_easy_setopt(m_pCURL, CURLOPT_TIMEOUT_MS, uiMsTimeout);
	curl_easy_setopt(m_pCURL, CURLOPT_CONNECTTIMEOUT_MS, uiMsTimeout);
	ResetResponse();
	m_dwLastHttpError = 0;

	// Proccess request
	if (m_pCURL != NULL)
	{
		sURL.Replace(_T(" "), _T("%20"));
		curl_easy_setopt(m_pCURL, CURLOPT_URL, sURL);
		curl_res = curl_easy_perform(m_pCURL);
	}

	// Errors ?

	switch (curl_res)
	{
	case CURLE_PARTIAL_FILE:
		// Ojo nuestros equipos devuelven este error porque indican un
		// content-length incorrecto. Pero en realidad significa otra cosa.
		// Hay que obtener el cdigo de error y simular que se obtuvo la respuesta
		// correcta por el socket.
	case CURLE_OK:
		m_pReadPosition = m_stChunk.memory;
		
		// Obtener el codigo de error devuelto por el servidor
		curl_easy_getinfo(m_pCURL, CURLINFO_RESPONSE_CODE, &m_dwLastHttpError);

		iResult = m_dwLastHttpError;
		if ((m_dwLastHttpError / 100) != 2)		
		{
			iResult = -2; 
			TRACE("HTTPClientCURL Error: %d \n",m_dwLastHttpError);
		}
		else
		{
			TRACE("HTTPClientCURL OK \n");
		}
		break;
	default:
		// Se han producido errores que no son HTTP (socket, buffer, etc)
		iResult = -1;
		m_dwLastHttpError = curl_res;
		TRACE("HTTPClientCURL Error : %d\n", m_dwLastHttpError);
		break;
	}

	return iResult;
}

int CMyHTTPClientCURL::HttpReadResponse(CString &strResponse)
{
	int iRes = 0;
	LPTSTR lpszStart = NULL;
	LPTSTR lpszEnd = NULL;

	if (m_pCURL == NULL) 
	{
		return -1;
	}

	strResponse.Empty();

	if (m_pReadPosition != NULL)
	{
		bool bContinue = false;

		do
		{
			strResponse.Empty();
			
			// Elimina los primeros saltos de linea
			lpszStart = m_pReadPosition;
			while ( (lpszStart != NULL) &&
					((*lpszStart == '\x0D') || (*lpszStart == '\x0A')) )
			{
				lpszStart++;
			}

			if (lpszStart == NULL)
			{	// No hay nada !
				break;
			}

			// Elimina los saltos de linea del final
			LPTSTR lpszEnd1 = strstr(lpszStart, _T("\x0D"));
			LPTSTR lpszEnd2 = strstr(lpszStart, _T("\x0A"));
			if (lpszEnd1 > lpszEnd2)
			{
				lpszEnd = lpszEnd1;
			}
			else
			{
				lpszEnd = lpszEnd2;
			}
			if(lpszEnd == NULL) 
			{
				strResponse.SetString(lpszStart);
				strResponse.TrimRight(0x0A);
				strResponse.TrimRight(0x0D);
				m_pReadPosition=NULL;
			}
			else if (lpszEnd > lpszStart)
			{
				strResponse.SetString(lpszStart, (int) (lpszEnd - lpszStart));
				
				strResponse.TrimRight(0x0A);
				strResponse.TrimRight(0x0D);
				
				m_pReadPosition = lpszEnd;
			}
			else
			{
				break;	// No hay nada mas
			}
			
			bContinue = (strResponse.IsEmpty() ? TRUE : ( strResponse.GetAt(0) == '<'));
		} 
		while (bContinue == true);
	}
	
	if (strResponse.IsEmpty() == true)
	{
		iRes = -1;
	}

	return iRes;
}

void CMyHTTPClientCURL::CloseRequest()
{
	ResetResponse();
}

int CMyHTTPClientCURL::CloseSession()
{
	ResetResponse();
	curl_easy_reset(m_pCURL);
	
	return 0;
}

void *CMyHTTPClientCURL::myrealloc(void *ptr, size_t size)
{
	/* There might be a realloc() out there that doesn't like reallocing
	 NULL pointers, so we take care of it here */
	if(ptr)
		return realloc(ptr, size);
	else
		return malloc(size);
}

size_t CMyHTTPClientCURL::WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data)
{
	size_t realsize = size * nmemb;
	struct MemoryStruct *mem = (struct MemoryStruct *)data;

	mem->memory = (char *)myrealloc(mem->memory, mem->size + realsize + 1);
	if (mem->memory)
	{
		memcpy(&(mem->memory[mem->size]), ptr, realsize);
		mem->size += realsize;
		mem->memory[mem->size] = 0;
	}
	return realsize;
}

void CMyHTTPClientCURL::ResetResponse()
{
	if(m_stChunk.memory)
	{
		free(m_stChunk.memory);
	}

	m_stChunk.memory = NULL; /* we expect realloc(NULL, size) to work */
	m_stChunk.size = 0;    /* no data at this point */
	m_pReadPosition = NULL;
}